--    cr2hdr developped by a1ex
--    http://www.magiclantern.fm/forum/index.php?topic=7139.0
--    Plugin developped for Magic Lantern
--    Add "cr2hdr" to export
--
--    Metadata added by akry
--
--    Copyright (C) <2014>  <kichetof kichetof@gmail.com>
--
--    This program is free software; you can redistribute it and/or modify
--    it under the terms of the GNU General Public License as published by
--    the Free Software Foundation; either version 2 of the License, or
--    (at your option) any later version.
--
--    This program is distributed in the hope that it will be useful,
--    but WITHOUT ANY WARRANTY; without even the implied warranty of
--    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--    GNU General Public License for more details.
--
--    You should have received a copy of the GNU General Public License along
--    with this program; if not, write to the Free Software Foundation, Inc.,
--    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

local LrBinding = import 'LrBinding'
local LrColor = import 'LrColor'
local LrDialogs = import 'LrDialogs'
local LrView = import "LrView"

local bind = LrView.bind

local MLDialogs = {}

-- Update optional arguments when --fast is selected
function MLDialogs.updateArguments(propertyTable)
	if propertyTable.shortcut_fast then
		propertyTable.interp_method = 1
		propertyTable.chroma_smooth_method = 0
		propertyTable.use_fullres = 0
		propertyTable.use_alias_map = 0
		propertyTable.use_stripe_fix = 0
	else
		propertyTable.interp_method = 0
		propertyTable.chroma_smooth_method = 2
		propertyTable.use_fullres = 1
		propertyTable.use_alias_map = 1
		propertyTable.use_stripe_fix = 1
	end
end

function MLDialogs.settings(f, propertyTable)
	local widthText = 300
	local widthSpacer = 5
	local fillRadio = 0.2
	
	propertyTable:addObserver("shortcut_fast", MLDialogs.updateArguments)
	
	return {
		{
			title = LOC "$$$/ML/ExportManager/Title=Magic Lantern Dual ISO",
			f:row {
				f:picture {
					value = _PLUGIN:resourceId("logo.png"),
				},
			},
			f:row {
				f:push_button {
					title = LOC "$$$/ML/ExportManager/MLTopic=Magic Lantern Dual ISO thread",
					action = function()
						LrHttp.openUrlInBrowser("http://www.magiclantern.fm/forum/index.php?topic=7139.0")
					end,					
				},
				f:push_button {
					title = LOC "$$$/ML/ExportManager/MLDocTech=Technical doc",
					action = function()
						LrHttp.openUrlInBrowser("http://acoutts.com/a1ex/dual_iso.pdf")
					end,					
				},
				f:spacer {
					height = 20
				},
				
			},
			f:row {
				f:static_text {
					title = LOC "$$$/ML/ExportManager/Description=This plugin allows to convert a Dual ISO picture with cr2hdr (thanks a1ex) from Magic Lantern.\n\nHow it works :\n\n- Export picture to temp folder\n- Add -dualiso suffix to picture\n- Convert picture with cr2hdr\n- Import converted picture into this LR Catalog\n- Add DualISO to the label\n- Add converted picture to the collection of the original",
				},
			},
		},
		{
			title = LOC "$$$/ML/ExportManager/CRTitle=cr2hdr optional arguments",
			visible = false,
			f:tab_view {
				value = "CRTab1",
				fill_horizontal = 1,
				f:tab_view_item {
					title = "Shortcuts",
					identifier = "CRTab1",
					f:row {
						f:checkbox {
							size = "small",
							text_color = LrColor("gray"),
							title = "--fast",
							tooltip = "disable most postprocessing steps (fast, but low quality)\n(--mean23, --no-cs, --no-fullres, --no-alias-map, --no-stripe-fix, --no-bad-pix)",
							value = bind 'shortcut_fast',
							checked_value = true,
							unchecked_value = false,
						},
					},
				},
				f:tab_view_item {
					title = "Interpolation methods",
					identifier = "CRTab2",
					f:row {
						f:radio_button {
							size = "small",
							text_color = LrColor("gray"),
							title = "--amaze-edge",
							tooltip = "use a temporary demosaic step (AMaZE) followed by edge-directed interpolation (default)",
							value = bind 'interp_method',
							checked_value = 0,
						},
					},
					f:row {
						f:radio_button {
							size = "small",
							text_color = LrColor("gray"),
							title = "--mean23",
							tooltip = "average the nearest 2 or 3 pixels of the same color from the Bayer grid (faster)",
							value = bind 'interp_method',
							checked_value = 1,
						},
					},
				},
				f:tab_view_item {
					title = "Chroma smoothing",
					identifier = "CRTab3",
					f:row {
						f:radio_button {
							size = "small",
							text_color = LrColor("gray"),
							title = "--no-cs",
							tooltip = "disable chroma smoothing",
							value = bind 'chroma_smooth_method',
							checked_value = 0,
						},
						f:radio_button {
							size = "small",
							text_color = LrColor("gray"),
							title = "--cs2x2",
							tooltip = "apply 2x2 chroma smoothing in noisy and aliased areas (default)",
							value = bind 'chroma_smooth_method',
							checked_value = 2,
						},
						f:radio_button {
							size = "small",
							text_color = LrColor("gray"),
							title = "--cs3x3",
							tooltip = "apply 3x3 chroma smoothing in noisy and aliased areas",
							value = bind 'chroma_smooth_method',
							checked_value = 3,
						},
						f:radio_button {
							size = "small",
							text_color = LrColor("gray"),
							title = "--cs5x5",
							tooltip = "apply 5x5 chroma smoothing in noisy and aliased areas",
							value = bind 'chroma_smooth_method',
							checked_value = 5,
						},
					},
				},
				f:tab_view_item {
					title = "Bad pixel handling",
					identifier = "CRTab4",
					f:row {
						f:radio_button {
							size = "small",
							text_color = LrColor("gray"),
							title = "--no-bad-pix",
							tooltip = "disable bad pixel fixing (try it if you shoot stars)",
							value = bind 'fix_bad_pixels',
							checked_value = 3,
						},
						f:radio_button {
							size = "small",
							text_color = LrColor("gray"),
							title = "--bad-pix",
							tooltip = "",
							value = bind 'fix_bad_pixels',
							checked_value = 1,
						},
						f:radio_button {
							size = "small",
							text_color = LrColor("gray"),
							title = "--really-bad-pix",
							tooltip = "aggressive bad pixel fix, at the expense of detail and aliasing",
							value = bind 'fix_bad_pixels',
							checked_value = 2,
						},
					},
					f:spacer {},
					f:row {
						f:checkbox {
							size = "small",
							text_color = LrColor("gray"),
							title = "--black-bad-pix",
							tooltip = "mark all bad pixels as black (for troubleshooting)",
							value = bind 'debug_bad_pixels',
							checked_value = true,
							unchecked_value = false,
						},
					},
				},
				f:tab_view_item {
					title = "Other postprocessing steps",
					identifier = "CRTab5",
					f:row {
						f:radio_button {
							fill_horizontal = fillRadio,
							size = "small",
							text_color = LrColor("gray"),
							title = "--no-fullres",
							tooltip = "disable full-resolution blending",
							value = bind 'use_fullres',
							checked_value = 0,
						},
						f:radio_button {
							fill_horizontal = fillRadio,
							size = "small",
							text_color = LrColor("gray"),
							title = "--fullres",
							tooltip = "",
							value = bind 'use_fullres',
							checked_value = 1,
						},
					},
					f:row {
						f:radio_button {
							fill_horizontal = fillRadio,
							size = "small",
							text_color = LrColor("gray"),
							title = "--no-alias-map",
							tooltip = "disable alias map, used to fix aliasing in deep shadows",
							value = bind 'use_alias_map',
							checked_value = 0,
						},
						f:radio_button {
							fill_horizontal = fillRadio,
							size = "small",
							text_color = LrColor("gray"),
							title = "--alias-map",
							tooltip = "",
							value = bind 'use_alias_map',
							checked_value = 1,
						},
					},
					f:row {
						f:radio_button {
							fill_horizontal = fillRadio,
							size = "small",
							text_color = LrColor("gray"),
							title = "--no-stripe-fix",
							tooltip = "disable horizontal stripe fix",
							value = bind 'use_stripe_fix',
							checked_value = 0,
						},
						f:radio_button {
							fill_horizontal = fillRadio,
							size = "small",
							text_color = LrColor("gray"),
							title = "--stripe-fix",
							tooltip = "",
							value = bind 'use_stripe_fix',
							checked_value = 1,
						},
					},
				},
				f:tab_view_item {
					title = "DNG compression",
					identifier = "CRTab6",
					f:row {
						f:radio_button {
							size = "small",
							text_color = LrColor("gray"),
							title = "--no-compress",
							tooltip = "Lossless DNG compression",
							value = bind 'compress',
							checked_value = 0,
						},
						f:radio_button {
							size = "small",
							text_color = LrColor("gray"),
							title = "--compress",
							tooltip = "Lossless DNG compression",
							value = bind 'compress',
							checked_value = 1,
						},
						f:radio_button {
							size = "small",
							text_color = LrColor("gray"),
							title = "--compress-lossy",
							tooltip = "Lossy DNG compression (be careful, may destroy shadow detail)",
							value = bind 'compress',
							checked_value = 2,
						},
					},
				},
			},
		},
		{
			title = LOC "$$$/ML/ExportManager/ASTitle=Advanced settings",
			visible = false,
			f:row {
				f:static_text {
					alignment = "right",
					width = widthText,
					title = LOC "$$$/ML/ExportManager/ASSuffix=Suffix for output dual iso file :",
				},
				f:spacer {
					width = widthSpacer,
				},
				f:static_text {
					size = "small",
					text_color = LrColor("gray"),
					title = "IMG_1234-",
				},
				f:edit_field {
					value = bind 'suffix',
					width = 60,
					
				},
				f:static_text {
					size = "small",
					text_color = LrColor("gray"),
					title = ".dng",
				},
			},
			f:row {
				f:static_text {
					alignment = "right",
					width = widthText,
					title = LOC "$$$/ML/ExportManager/Keywords=Add keywords :",
				},
				f:spacer {
					width = widthSpacer,
				},
				
				f:edit_field {
					value = bind 'keywords',
					width = 120,
					placeholder_string = "Dual ISO, cr2hdr, ...",
				},
				f:static_text {
					size = "small",
					text_color = LrColor("gray"),
					title = LOC "$$$/ML/ExportManager/KeywordsInfo=comma-separated",
				},
			},
			f:row {
				f:static_text {
					width = widthText,
					alignment = "right",
					title = LOC "$$$/ML/ExportManager/ASLabel=Define the color label for Dual ISO file :",
				},
				f:spacer {
					width = widthSpacer,
				},
				f:popup_menu {
					items = bind 'labels',
					value = bind "label",
					size = 'small'
				},
				f:static_text {
					size = "small",
					text_color = LrColor("gray"),
					title = LOC "$$$/ML/ExportManager/ASLabelInfo=Leave blank to not add color label",
				},
			},
			f:row {

				f:static_text {
					width = widthText,
					alignment = "right",
					title = LOC "$$$/ML/ExportManager/ASSubfolder=Add the converted file to this subfolder :",
				},
				f:spacer {
					width = widthSpacer,
				},
				f:edit_field {
					value = bind 'subfolder',
					width = 100,
					placeholder_string = LOC "$$$/ML/ExportManager/ASSubfolderPL=Parent",
				},
				f:static_text {
					text_color = LrColor("gray"),
					alignment = "right",
					title = LOC "$$$/ML/ExportManager/ASSubfolderInfo=leave blank to reimport into the same folder",
				},
			},
			f:row {
				f:static_text {
					width = widthText,
					alignment = "right",
					title = LOC "$$$/ML/ExportManager/ASCollection=Add Dual ISO file to collection ?",
				},
				f:spacer {
					width = widthSpacer,
				},
				f:checkbox {
					size = "small",
					text_color = LrColor("gray"),
					title = LOC "$$$/ML/ExportManager/ASCollectionInfo=Uncheck to not add to the original picture collection",
					value = bind 'addToCollection',
					checked_value = true,
					unchecked_value = false,
				},
			},
			f:row {
				f:static_text {
					width = widthText,
					alignment = "right",
					title = LOC "$$$/ML/ExportManager/ASLog=Keep log file after conversion ?",
				},
				f:spacer {
					width = widthSpacer,
				},
				f:checkbox {
					size = "small",
					text_color = LrColor("gray"),
					title = LOC "$$$/ML/ExportManager/ASLogInfo=Uncheck to remove",
					value = bind 'saveOutput',
					checked_value = true,
					unchecked_value = false,
				},
			},
			f:row {
				f:static_text {
					width = widthText,
					alignment = "right",
					title = LOC "$$$/ML/ExportManager/ASSummary=Show summary message after conversion ?",
				},
				f:spacer {
					width = widthSpacer,
				},
				f:checkbox {
					size = "small",
					text_color = LrColor("gray"),
					title = LOC "$$$/ML/ExportManager/ASSummaryInfo=Uncheck to hide",
					value = bind 'showSummary',
					checked_value = true,
					unchecked_value = false,
				},
			},
		},
	}
end

return MLDialogs