--    cr2hdr developped by a1ex
--    http://www.magiclantern.fm/forum/index.php?topic=7139.0
--    Plugin developped for Magic Lantern
--    Add "cr2hdr" to export
--
--    Metadata added by akry
--
--    Copyright (C) <2014>  <kichetof kichetof@gmail.com>
--
--    This program is free software; you can redistribute it and/or modify
--    it under the terms of the GNU General Public License as published by
--    the Free Software Foundation; either version 2 of the License, or
--    (at your option) any later version.
--
--    This program is distributed in the hope that it will be useful,
--    but WITHOUT ANY WARRANTY; without even the implied warranty of
--    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--    GNU General Public License for more details.
--
--    You should have received a copy of the GNU General Public License along
--    with this program; if not, write to the Free Software Foundation, Inc.,
--    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

local LrApplication = import 'LrApplication'
local LrLogger = import 'LrLogger'
local LrTasks = import 'LrTasks'
local MLDialogs = require 'MLDialogs'
local MLProcess = require 'MLProcess'

function getKeyForValue(t, value)
	for k,v in pairs(t) do
		if v==value then return k end
	end
	return nil
end

function getLabelColor(propertyTable)
	local catalog = LrApplication:activeCatalog()
	local labelColor = catalog:getLabelMapToColorName()

	propertyTable.labels = { 
		{ value = "", title = "" },
		{ value = getKeyForValue(labelColor, "red"), title = getKeyForValue(labelColor, "red") },
		{ value = getKeyForValue(labelColor, "green"), title = getKeyForValue(labelColor, "green") },
		{ value = getKeyForValue(labelColor, "yellow"), title = getKeyForValue(labelColor, "yellow") },
		{ value = getKeyForValue(labelColor, "blue"), title = getKeyForValue(labelColor, "blue") },
		{ value = getKeyForValue(labelColor, "purple"), title = getKeyForValue(labelColor, "purple") },
	}

end

-- var for log
mlLog = LrLogger('MLDualISO')
mlLog:enable("print") -- replace print with logfile to log into a file in ~/Documents on Mac OS, or My Documents on Windows
--mlLog:disable() -- uncomment it to disable log (only on release version)
	
exportServiceProvider = {}

exportServiceProvider.canExportVideo = false

exportServiceProvider.exportPresetFields = {
	{ key = 'suffix',           	default = "dualiso" },
	{ key = 'keywords',				default = "Dual ISO" },
	{ key = 'label',            	default = "" },
	{ key = 'labels',				default = {} },
	{ key = 'saveOutput',       	default = true },
	{ key = 'addToCollection',  	default = true },
	{ key = 'subfolder',        	default = "" },
	{ key = 'showSummary',        	default = true },
	{ key = 'shortcut_fast',		default = false },
	{ key = 'interp_method',		default = 0 },		--0 amaze, 1 mean23
	{ key = 'chroma_smooth_method',	default = 2 },		--2 cs2, 3 cs3, 5 cs5, 0 no-cs
	{ key = 'fix_bad_pixels',		default = 0 },		--1 bad, 2 really bad, 0 no bad
	{ key = 'debug_bad_pixels',		default = false },
	{ key = 'use_fullres', 			default = 1 },		--0 no full, 1 full
	{ key = 'use_alias_map',		default = 1 },		--0 no map, 1 map
	{ key = 'use_stripe_fix',		default = 1 },		--0 no stripe, 1 stripe
	{ key = 'compress',				default = 0 },		--0 no compress, 1 compress, 2 compress lossy
}

exportServiceProvider.hideSections = { "exportLocation", "fileNaming", "fileSettings", "imageSettings", "outputSharpening", "metadata", "watermarking", "video" }

function exportServiceProvider.startDialog(propertyTable)

	LrTasks.startAsyncTask(function ()
		getLabelColor(propertyTable)
	end)

end

function exportServiceProvider.endDialog( propertyTable )

	propertyTable.LR_tokens = string.format("{{image_name}}-%s", propertyTable.suffix)
	propertyTable.LR_format = 'ORIGINAL'
	propertyTable.LR_export_colorSpace = "AdobeRGB"

end

function exportServiceProvider.updateExportSettings(exportSettings)

	mlLog:debug("I'm called before exportServiceProvider.processRenderedPhotos")

end

function exportServiceProvider.processRenderedPhotos(functionContext, exportContext)
	
	return MLProcess.process(functionContext, exportContext)

end

function exportServiceProvider.sectionsForTopOfDialog(f, propertyTable)

	return MLDialogs.settings(f, propertyTable)

end

return exportServiceProvider