--    cr2hdr developped by a1ex
--    http://www.magiclantern.fm/forum/index.php?topic=7139.0
--    Plugin developped for Magic Lantern
--    Add "cr2hdr" to export
--
--    Metadata added by akry
--
--    Copyright (C) <2014>  <kichetof kichetof@gmail.com>
--
--    This program is free software; you can redistribute it and/or modify
--    it under the terms of the GNU General Public License as published by
--    the Free Software Foundation; either version 2 of the License, or
--    (at your option) any later version.
--
--    This program is distributed in the hope that it will be useful,
--    but WITHOUT ANY WARRANTY; without even the implied warranty of
--    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--    GNU General Public License for more details.
--
--    You should have received a copy of the GNU General Public License along
--    with this program; if not, write to the Free Software Foundation, Inc.,
--    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

local LrApplication = import 'LrApplication'
local LrColor = import 'LrColor'
local LrDialogs = import 'LrDialogs'
local LrErrors = import 'LrErrors'
local LrExportSession = import 'LrExportSession'
local LrFileUtils = import 'LrFileUtils'
local LrFunctionContext = import 'LrFunctionContext'
local LrHttp = import 'LrHttp'
local LrLogger = import 'LrLogger'
local LrPathUtils = import 'LrPathUtils'
local LrProgressScope = import 'LrProgressScope'
local LrTasks = import 'LrTasks'
local LrView = import "LrView"

local MLProcess = {}

-- Check for Mac dependencies
function MLProcess.checkDependencies()
	return "exec which dcraw exiftool"
end

-- Fill empty table for summary message
function MLProcess.checkTab (tab, err)
	local msg = err ~= 1 and LOC "$$$/ML/Publish/Finish/NoImage=No image" or LOC "$$$/ML/Publish/Finish/NoError=No error"
	return #tab == 0 and table.insert(tab, { ["title"] = msg, ["value"] = "" }) or tab
end

-- Return command line for cr2hdr
function MLProcess.generateCmd(exportSettings, pluginPath, filePath, output, args)

	local exec = "cr2hdr"
	local gt = " > "
	local cmd = WIN_ENV and 'start /D "%s\\bin" /B /WAIT /BELOWNORMAL %s.exe%s "%s"' or 'exec "%s/bin/%s"%s"" "%s"'
	local log = '%s%s"%s"'
	local escape = WIN_ENV and '"%s"' or '%s'
	
	return string.format(escape,
		string.format(log, 
			string.format(cmd,
				pluginPath,
				exec,
				args,
				filePath
			),
			gt,
			output)
	)
	
end

-- remove trailing and leading spaces
function MLProcess.trim(s)
	return (s:gsub("^%s*(.-)%s*$", "%1"))
end

function MLProcess.split(myString, inSplitPattern, outResults)

	if not outResults then
	
		outResults = { }
	
	end
	
	local theStart = 1
	local theSplitStart, theSplitEnd = string.find(myString, inSplitPattern, theStart)
	
	while theSplitStart do
	
		table.insert( outResults, string.sub( myString, theStart, theSplitStart-1 ) )
		theStart = theSplitEnd + 1
		theSplitStart, theSplitEnd = string.find( myString, inSplitPattern, theStart )
	
	end
	
	table.insert(outResults, string.sub(myString, theStart))
	
	return outResults
  
end

-- parse logfile to read info
function MLProcess.parseLogFileToArray(filename)

	local logFileOpen = assert(io.open(filename, 'r'))
	
	local logfile = logFileOpen:read("*all")
	logFileOpen:close() 
	
	info = {}
	info_index = {}
	
	-- parse cr2hdr output for "xxx : yyy" and store keys/values in temp_t
	for k, v in string.gmatch(logfile, '%s*([%w%s-]+)%s*:%s*([%S ]+)%s*') do
	
		k = MLProcess.trim(k)

		-- there are duplicating keys in output (for instance, "Dynamic range")
		if info[k] ~= nil then
		
			local t = k
			info_index[t] = info_index[t] + 1
			k = k .. info_index[t]
			
		else
		 
			info_index[k] = 1
		
		end
		
		info[k] = MLProcess.trim(v)
		
	end
	
	return info
		
end

-- fill metada
function MLProcess.fillMetadata(photo, meta)

	-- exit function if metadata array is empty
	if meta == nil then
	
		do return end
	
	end
	
	local metaString1
	local metaString2
	
	if meta['ISO difference'] ~= nil then
		metaString1 = meta['ISO difference']:match("(%S+ EV)")
		photo:setPropertyForPlugin (_PLUGIN, 'ISOdifference', metaString1)
	end
	
	if meta['Dynamic range2'] ~= nil then
		metaString1 = meta['Dynamic range2']:match("(%S+ EV)")
		photo:setPropertyForPlugin (_PLUGIN, 'DynamicRange', metaString1)
	end
	
	if meta['Dynamic range'] ~= nil then
		metaString1, metaString2 = meta['Dynamic range']:match("(%S+) %(%+%) (%S+)")
		photo:setPropertyForPlugin (_PLUGIN, 'DynamicRangeOrig1', metaString1 .. ' EV')
		photo:setPropertyForPlugin (_PLUGIN, 'DynamicRangeOrig2', metaString2 .. ' EV')
	end
	
	if meta['Semi-overexposed'] ~= nil then
		metaString1 = meta['Semi-overexposed']:match("(%S+%%)")
		photo:setPropertyForPlugin (_PLUGIN, 'SemiOverexposed', metaString1)
	end
	
	if meta['Deep shadows'] ~= nil then
		metaString1 = meta['Deep shadows']:match("(%S+%%)")
		photo:setPropertyForPlugin (_PLUGIN, 'DeepShadows', metaString1)
	end
	
	
	if meta['Noise level'] ~= nil then
		metaString1, metaString2 = meta['Noise level']:match("(%S+) .+, ideally (%S+)")
		photo:setPropertyForPlugin (_PLUGIN, 'NoiseLevel', metaString1)
		photo:setPropertyForPlugin (_PLUGIN, 'NoiseLevelIdeal', metaString2)
	end
	
	if meta['ISO overlap'] ~= nil then
		metaString1 = meta['ISO overlap']:match("(%S+ EV)")
		photo:setPropertyForPlugin (_PLUGIN, 'ISOoverlap', metaString1)
	end

end

function MLProcess.process(functionContext, exportContext)
	
	-- var for summary message
	local summary = {
		["success"] = {},
		["nodual"] = {},
		['dual'] = {},
		['format'] = {},
		["fail"] = {},
		["errorMsg"] = {}
	}
	
	

	functionContext:addCleanupHandler(
		function(status, message)
	        if not status then
	        
	              LrDialogs.message("~ There has been an error ~", string.format("Please report the problem to Magic Lantern forum\n\n%s", message), "critical")
	        
	        end
	    end
    )
	if MAC_ENV then
		
		local cmd = MLProcess.checkDependencies()
		isDependOk = LrTasks.execute(cmd)

		if isDependOk ~= 0 then
			
			LrDialogs.message(LOC "$$$/ML/Dependencies/ErrorTitle=~ Dependencies error ~", LOC "$$$/ML/Dpendencies/Error=This plugin (cr2hdr) need dcraw and exiftool.^n^nPlease install it before run this plugin")
			do return end
			
		end
	end
	
	local exportSession = exportContext.exportSession
	local exportSettings = exportContext.propertyTable
	local nPhotos = exportSession:countRenditions()
	local progressScope = exportContext:configureProgress {
		title = nPhotos > 1 and LOC("$$$/ML/Publish/Progress=Exporting ^1 photos to cr2hdr", nPhotos) or LOC "$$$/ML/Publish/Progress/One=Exporting one photo to cr2hdr",
	}

	for i, rendition in exportContext:renditions { stopIfCanceled = true } do

		local fileFormat = rendition.photo:getRawMetadata("fileFormat")
		if fileFormat == "RAW" or fileFormat == "DNG" then
			
			progressScope:setPortionComplete((i - 1) / nPhotos)
			
			if not rendition.wasSkipped then
			
				local success, pathOrMessage = rendition:waitForRender()
				progressScope:setPortionComplete((i - 0.5) / nPhotos)
				
				local originalFilePath = rendition.photo.path
				mlLog:debugf("originalFilePath %s", originalFilePath)
				-- original filename to export (exemple img_1234.cr2)
				local originalName = LrPathUtils.leafName(originalFilePath)
				mlLog:debugf("originalName %s", originalName)
				-- original path to reimport
				local originalPath = LrPathUtils.parent(originalFilePath)
				
				if exportSettings.subfolder ~= "" then
				
					originalPath = LrPathUtils.child(originalPath, exportSettings.subfolder)
					local createSubFolder = LrFileUtils.createAllDirectories(originalPath)
					mlLog:debugf("createSubFolder : %s", createSubFolder)
					
				end
				
				mlLog:debugf("originalPath %s", originalPath)
				-- get original collection
				local originalCollection = rendition.photo:getContainedCollections()
				mlLog:debugf("originalCollection %s", originalCollection)
				-- export path & filename (exemple ~/images/dualiso_1234.cr2)
				local filePath = assert(pathOrMessage)
				mlLog:debugf("filePath %s", filePath)
				-- export path & filename for DNG file
				local filePathDNG = LrPathUtils.replaceExtension(filePath, "DNG")
				mlLog:debugf("filePathDNG %s", filePathDNG)
				-- the filename for reimport to LR
				local importFileDNG = LrPathUtils.leafName(filePathDNG)
				mlLog:debugf("importFileDNG %s", importFileDNG)
				-- the filename & path to reimport into LR
				local importDNG = LrPathUtils.child(originalPath, importFileDNG)
				-- possibility to bypass same name; disable to see already treated message
				--local importDNG = LrFileUtils.chooseUniqueFileName(importDNG)
				mlLog:debugf("importDNG %s", importDNG)
				-- export path & filename for txt output command
				local output = LrPathUtils.replaceExtension(importDNG, "txt")
				mlLog:debugf("output %s", output)
				
				if LrFileUtils.exists(importDNG) == false then
				
					if progressScope:isCanceled() then break end
					
					if success then
					
						-- plugin path
						local pluginPath = _PLUGIN.path
						
						local args = string.format('%s%s%s%s%s%s%s%s%s', 
							(exportSettings.shortcut_fast == true) and " --fast" or "",
							(exportSettings.interp_method == 0) and " --amaze-edge" or " --mean23",
							(exportSettings.chroma_smooth_method == 0) and " --no-cs" or string.format(" --cs%dx%d", exportSettings.chroma_smooth_method, exportSettings.chroma_smooth_method),
							(exportSettings.fix_bad_pixels == 0) and " --no-bad-pix" or ((exportSettings.fix_bad_pixels == 1) and " --bad-pix" or " --really-bad-pix"),
							(exportSettings.debug_bad_pixels == true) and " --black-bad-pix" or "",
							(exportSettings.use_fullres == 0) and " --no-fullres" or " --fullres",
							(exportSettings.use_alias_map == 0) and " --no-alias-map" or " --alias-map",
							(exportSettings.use_stripe_fix == 0) and " --no-stripe-fix" or " --stripe-fix",
							(exportSettings.compress == 1) and " --compress" or ((exportSettings.compress == 2) and "--compress-lossy" or ""))
						mlLog:debugf("arguments : %s", args)

						local command = MLProcess.generateCmd(exportSettings, pluginPath, filePath, output, args)
						mlLog:debugf("command : %s", command)
						result = LrTasks.execute(command)
						
						if result == 0 then

							if LrFileUtils.exists(filePathDNG) ~= false then
							
								local success, moveError = LrFileUtils.copy(filePathDNG, importDNG)
								if success then
								
									local catalog = LrApplication:activeCatalog()
									local importDualiso = catalog:withWriteAccessDo('Import from ML DualISO',
										function(context)
											catalog:addPhoto(importDNG, rendition.photo, 'above')
											local photo = catalog:findPhotoByPath(importDNG)
											
											photo:setRawMetadata('label', exportSettings.label)
											
											if exportSettings.keywords ~= '' then
											
												local keywords = MLProcess.split (exportSettings.keywords, ",")
												local keyword
												
												for i, key in ipairs (keywords) do
												
													key = MLProcess.trim(key)
													
													if key ~= "" then
													
														keyword = catalog:createKeyword(key, nil, true, nil, true)
														photo:addKeyword(keyword)
														
													end
													
												end
												
											end
											
											local metadata = MLProcess.parseLogFileToArray(output)
											MLProcess.fillMetadata(photo, metadata)
											
											if exportSettings.addToCollection and next(originalCollection) ~= nil then
	
												for k, v in pairs(originalCollection) do
												
													v:addPhotos({ photo })
													
												end
												
											end
											
											if not exportSettings.saveOutput then
												
												LrFileUtils.moveToTrash(output)
											
											end
											
											summary.success[#summary.success+1] = { ["title"] = originalName, ["value"] = "" }
											
										end,
										{timeout = 30}
									)
									
									if importDualiso ~= "executed" then
										
										summary.errorMsg[#summary.errorMsg+1] = { ["title"] = LOC "$$$/ML/Import/Error=Unable to import the dual iso file!^n^nPlease manually synchronise your folder", ["value"] = "" }
										
									end
								else
								
									summary.errorMsg[#summary.errorMsg+1] = { ["title"] = LOC ("$$$/ML/Move/Error=Unable to copy the file^n^nfrom:^n^1^nto:^n^2^nError:^n^3", filePathDNG, importDNG, moveError), ["value"] = "" }

								end
								
							else
								
								summary.nodual[#summary.nodual+1] = { ["title"] = originalName, ["value"] = "" }
								
							end
							
						else
							
							summary.errorMsg[#summary.errorMsg+1] =  { ["title"] = LOC("$$$/ML/Publish/Error=Unable to export ^1 to cr2hdr! Error ^2", originalName, result), ["value"] = "" }
							
							break
						
						-- end if cr2hdr
						end
						
					-- end if success	
					end
					
				else
				
					summary.dual[#summary.dual+1] = { ["title"] = originalName, ["value"] = "" }
					
				end
			-- rendition skip
			end
			
		else

			summary.format[#summary.format+1] = { ["title"] = LrPathUtils.leafName(rendition.photo.path), ["value"] = "" }
		
		-- end if not raw
		end
	
	-- end for
	end
	
	progressScope:done()
		
	-- Show final message
	mlLog:debugf("success: %d - nodual: %d - dual: %d - format: %d - fail: %d - errorMsg: %d",
		#summary.success,
		#summary.nodual,
		#summary.dual,
		#summary.format,
		#summary.fail,
		#summary.errorMsg
		)
	
	if exportSettings.showSummary then
	
		local v = LrView.osFactory()
		local totalImg = #summary.success
		local margin = 5
		local width = 600
	
		LrDialogs.presentModalDialog {
			title = LOC "$$$/ML/Publish/Finish=cr2hdr export finished",
			resizable = "vertically",
			cancelVerb = "< exclude >",
			contents = v:view {
				width = width,
				v:static_text {
					width = width,
					title = totalImg == 0 and LOC "$$$/ML/Publish/NoPhotoExported=No photo has been exported" or totalImg <= 1 and LOC("$$$/ML/Publish/NbPhotoExported=^1 photo has been exported with success", totalImg) or LOC("$$$/ML/Publish/NbPhotosExported=^1 photos have been exported with success", totalImg),
					alignment = "center",
					font = "<system/bold>",
				},
				v:spacer {
					height = 20
				},
				v:tab_view {
					value = "FinishTab1",
					fill_horizontal = 1,
					v:tab_view_item {
						title = LOC("$$$/ML/Publish/Finish/Success=Converted^1", (#summary.success > 0 and " (" .. #summary.success .. ")" or "")),
						identifier = "FinishTab1",
						v:simple_list {
							width = width,
							items = MLProcess.checkTab(summary.success, 0),
						},
					},
					v:tab_view_item {
						title = LOC("$$$/ML/Publish/Finish/noDual=No DualISO^1", (#summary.nodual > 0 and " (" .. #summary.nodual .. ")" or "")),
						identifier = "FinishTab2",
						v:simple_list {
							width = width,
							items = MLProcess.checkTab(summary.nodual, 0),
						},
					},
					v:tab_view_item {
						title = LOC("$$$/ML/Publish/Finish/AlreadyConvertTitle=Already treated^1", (#summary.dual > 0 and " (" .. #summary.dual .. ")" or "")),
						identifier = "FinishTab3",
						v:simple_list {
							width = width,
							items = MLProcess.checkTab(summary.dual, 0),
						},
					},
					v:tab_view_item {
						title = LOC("$$$/ML/Publish/Finish/NotRawTitle=Wrong format^1", (#summary.format > 0 and " (" .. #summary.format .. ")" or "")),
						identifier = "FinishTab4",
						v:simple_list {
							width = width,
							items = MLProcess.checkTab(summary.format, 0),
						},
					},
					v:tab_view_item {
						title = LOC("$$$/ML/Publish/Finish/Error=Error^1", (#summary.errorMsg > 0 and " (" .. #summary.errorMsg .. ")" or "")),
						identifier = "FinishTab5",
						v:simple_list {
							width = width,
							items = MLProcess.checkTab(summary.errorMsg, 1),
						},
					},
				},
			}
		}
	
	end
end

return MLProcess